// Copyright (c) 2014-2015 The Notify Authors. All rights reserved.
// Use of this source code is governed by the MIT license that can be
// found in the LICENSE file.

// +build !debug

package notify

func dbgprint(...interface{}) {}

func dbgprintf(string, ...interface{}) {}
